{ --------------------------------------------------------------------------- }
{                       The Game of JETAN -- Version 2.2                      }
{                            Copyright (c)1993 by                             }
{                          Silver Moon Software, Ltd.                         }
{                                                                             }
{     In the eleven volumes of his "John Carter of Mars" series, Edgar Rice   }
{  Burroughs, the master of adventure stories, tells of the many weird        }
{  peoples and customs of the dying red planet Mars, or "Barsoom." In one     }
{  volume, entitled "The Chessmen of Mars," Burroughs describes the game of   }
{  Jetan, or Barsoomian chess. Jetan is played with forty pieces on a board   }
{  of ten by ten squares. This computer program is designed to play Jetan     }
{  against a human opponent...                                                }
{     The central game-tree routines, such as playgame, findmax, and findmin, }
{  are adapted from Paul Sand, "Advanced Programming Techniques in Pascal,"   }
{  where versions of them appear in a program for the game of Reversi.        }
{     The quicksort routine is from Ellis Horowitz & Sartaj Sahni,            }
{  "Fundamentals of Data Structures in Turbo Pascal."                         }
{     Some details of the game tree search are adapted from the "Computer     }
{  Chess Compendium," David Levy, editor.                                     }
{     The graphics for board and chessmen are courtesy of Donald Pavia's      }
{  Turbo Sprite graphics routine library and Turbo Sprite Designer utility.   }
{                                                                             }
{     SOURCE CODE files for Jetan 2.2:                                        }
{         JETAN.PAS -- The main Jetan program                                 }
{         JETANSPR.P -- Contains adaptation of Pavia's Turbo Sprite routines  }
{         JETANGRF.P -- Contains graphics for Jetan board and pieces          }
{         JETAN1.P  -- Contains definition of legal moves for pieces          }
{ --------------------------------------------------------------------------- }
program jetan (input, output);
{$R-}    {Range checking off}
{$B-}    {Boolean complete evaluation off}
{$S-}    {Stack checking off}
{$I+}    {I/O checking on}
{$N-}    {No numeric coprocessor}
{$M 65500,16384,655360} {Turbo 3 default stack and heap}

Uses
  Crt, {Unit found in TURBO.TPL}
  Dos, {Unit found in TURBO.TPL}
  Turbo3, {Needed for read(kbd,ch) statements }
  Graph3; {Unit found in GRAPH3.TPU}

{$I jetanspr.p }
{$I jetangrf.p }

const
  spritetablename = 'JETAN.SCR';
  maxcrtcol = 30;
  maxmoves = 180;
  maxpieces = 20;
  maxchief = 5;
  minrank = 48;
  maxrank = 192;
  minfile = 3;
  maxfile = 12;
  initlookahead = 2;
  initlevel = 1;
  minlevel = 1;
  maxlevel = 9;
  squarenumsize = 255;
  leftmargin = 80;
  topmovewiny = 9;
  botmovewiny = 20;
  insufficientforce = 5;
  drawmovelimit = 10;
  ESC = #27;
  CR = #13;

  { Constants for use in calculating legal moves }
  up3left3 = -51; up3left2 = -50; up3left1 = -49;
    up3 = -48;
    up3right1 = -47; up3right2 = -46; up3right3 = -45;
  up2left3 = -35; up2left2 = -34; up2left1 = -33;
    up2 = -32;
    up2right1 = -31; up2right2 = -30; up2right3 = -29;
  up1left3 = -19; up1left2 = -18; up1left1 = -17;
    up1 = -16;
    up1right1 = -15; up1right2 = -14; up1right3 = -13;
  left3 = -3; left2 = -2; left1 = -1;
     right1 = 1; right2 = 2; right3 = 3;
  down1left3 = 13; down1left2 = 14; down1left1 = 15;
    down1 = 16;
    down1right1 = 17; down1right2 = 18; down1right3 = 19;
  down2left3 = 29; down2left2 = 30; down2left1 = 31;
    down2 = 32;
    down2right1 = 33; down2right2 = 34; down2right3 = 35;
  down3left3 = 45; down3left2 = 46; down3left1 = 47;
    down3 = 48;
    down3right1 = 49; down3right2 = 50; down3right3 = 51;

  { Indices for array of sprites }
  othoato = 1;     othoatb = 2;     bthoato = 3;     bthoatb = 4;
  opanthano = 5;   opanthanb = 6;   bpanthano = 7;   bpanthanb = 8;
  owarrioro = 9;   owarriorb = 10;  bwarrioro = 11;  bwarriorb = 12;
  opadwaro = 13;   opadwarb = 14;   bpadwaro = 15;   bpadwarb = 16;
  odwaro = 17;     odwarb = 18;     bdwaro = 19;     bdwarb = 20;
  ofliero = 21;    oflierb = 22;    bfliero = 23;    bflierb = 24;
  oprincesso = 25; oprincessb = 26; bprincesso = 27; bprincessb = 28;
  ochiefo = 29;    ochiefb = 30;    bchiefo = 31;    bchiefb = 32;
  emptyo = 33;     emptyb = 34;     cursoro = 35;    cursorb = 36;
  greenline = 37;

type
  linestring = string [maxcrtcol];
  shortstring = string[3];
  twobyte = record
    l,h : byte
  end;
  regpack = record
    ax,bx,cx,dx,bp,si,di,ds,es,flags : twobyte
  end;
  levelspan = minlevel..maxlevel;
  initmovetimearray = array [levelspan] of integer;
  truthchar = array [boolean] of char;
  truthbyte = array [boolean] of byte;
  truthstr  = array [boolean] of shortstring;
  boardgauge = array [3..12] of byte;
  charset = set of char;
  contents = (othoat, opanthan, owarrior, opadwar, odwar, oflier,
                oprincess, ochief,
              bthoat, bpanthan, bwarrior, bpadwar, bdwar, bflier,
                bprincess, bchief,
              empty, border);
  valuearray = array [contents] of integer;
  player = (orangepc,blackpc);
  squarenum = byte; { 10x10 board with three border rows all around }
  npieces = 0..maxpieces;
  chessmenarray = array [player,npieces] of squarenum;
  movelist = record
    nmoves : 0..maxmoves;
    movefrom, moveto : array [0..maxmoves] of squarenum;
    key : array [0..maxmoves] of integer
  end;
  board = record
    sq: array [squarenum] of contents;
    chessmen : chessmenarray;
    numchessmen : array [player] of npieces;
    possible : movelist;
    escapeyet : array [player] of boolean;
    chiefcaptor, princesscaptor : contents
  end;

const  { Typed constant declarations }
  initchessmen : chessmenarray =
  { Position of chessmen at start of game;        }
  { ordered to maximize speed of game tree search }
  ((0, 58, 53, 57, 54, 56, 76, 67, 74, 69, 73,
       72, 70, 71, 75, 68, 59, 52, 60, 51, 55),
   (0,197,202,198,201,199,179,188,181,186,182,
      183,185,184,180,187,196,203,195,204,200));
  blinkch : truthchar = (' ','*');
  movech : truthchar = ('x','-');
  { Value of chessmen by rank }
  value : valuearray =
  (300,100,200,200,700,600,5000,5000,
   300,100,200,200,700,600,5000,5000,
   0,0);
  { Gauge of concentric ranks and files on board }
  bdgauge : boardgauge = (1,2,3,4,5,5,4,3,2,1);
  { Maximum move time on each game level, 1-9 }
  initmovetime : initmovetimearray =
  (10,30,60,120,240,360,720,1200,maxint);

var
  ch : char;
  moves, movey : byte;
  level, lookahead, halfmaxint, movetime, drawin10 : integer;
  functionkey, fullthoatmove, messageon, soundon,
    rightsideup, changesides, invertingboard, blinker,
    jetanover, gameover, opening : boolean;
  chessclock, split : array[player] of real;
  inittime : real;
  tablefile : file;
  computer, human : player;
  tempbd : board;
  templist, gamelist : movelist;

procedure error (errormessage : linestring);
begin
  textmode(C80);
  writeln(errormessage);
  halt
 end; { error }

procedure beep (tone, waittime : integer);
begin
  if soundon then begin
    sound(tone);
    delay(waittime);
    nosound
  end
end; { beep }

function gettime : real;
var
  reg : regpack;
begin
  with reg do begin
    { Int 21H Fct 2CH  Get Time }
    ax.h := $2C;
    intr($21,Dos.Registers(reg));
    if 3600 * (cx.h + 1) < inittime then { clock has passed midnight }
      cx.h := cx.h + 24;
    gettime := 3600 * cx.h + 60 * cx.l + dx.h + 0.01 * dx.l - inittime
  end
end; { gettime }

procedure showclock (var pl: player);
{ Display player or computer chess clock at upper left of screen }
const
  clockline : truthbyte = (5,7);

procedure showminsec (minsec : integer);
begin
  case minsec of
    0..9   : write(':0',minsec:1);
    10..59 : write(':',minsec:2)
  end
end; { showminsec }

begin { showclock }
  gotoxy(1,clockline[(pl = orangepc) xor rightsideup]);
  if not invertingboard then begin
    chessclock[pl] := chessclock[pl] + gettime - split[pl];
    split[pl] := gettime
  end;
  write((trunc(chessclock[pl]) div 3600):2);
  showminsec((trunc(chessclock[pl]) mod 3600) div 60);
  showminsec(trunc(chessclock[pl]) mod 60)
end; { showclock }

function getkey (var ch: char; valid : charset;
                 var functionkey : boolean): char;
{ Get valid key typed at keyboard, no echo }
var
  ok : boolean;
  junk : char;
begin
  ok := false;
  repeat
    if keypressed then begin
      read(kbd,ch);
      if (ch = ESC) and keypressed then begin
        read(kbd,ch);
        functionkey := true
      end
      else
        functionkey := false;
      getkey := ch;
      ok := upcase(ch) in valid;
      if not ok then
        beep(440,250);
      while keypressed do
        read(kbd,junk)
    end
    else if not gameover then
      showclock(human)
  until ok;
  getkey := upcase(ch)
end; { getkey }

procedure movewindow;
begin
  gotoxy(1,movey)
end; { movewindow }

procedure messagewindow;
begin
  gotoxy(1,22)
end; { messagewindow }

procedure showmove (kfrom, kto : squarenum; c : contents; pl : player);
{ Display move in move window, in algebraic notation }
var
  i : byte;

function algebraic (k : squarenum): linestring;
var
  temp : linestring;
begin
  str((12 - k div 16):1,temp);
  algebraic := concat(char(k mod 16 + 62), temp)
end; { algebraic }

procedure movelineup (var i : byte);
var
  reg : regpack;
  j : byte;
begin
  with reg do begin
    for j := 1 to 10 do begin
      gotoxy(j,i+1);
      { Int 10H Fct 08H  Read Character and Attribute at Cursor }
      ax.h := $08;
      bx.h := $00;
      intr($10,Dos.Registers(reg));
      gotoxy(j,i);
      { Int 10H Fct 09H  Write Character and Attribute at Cursor }
      ax.h := $09;
      { ax.l already defined }
      bx.h := $00;
      bx.l := lightgreen;
      cx.h := $00;
      cx.l := $01;
      intr($10,Dos.Registers(reg))
    end
  end
end; { movelineup }

begin { showmove }
  movewindow;
  if (movey >= botmovewiny - 1) and
     (moves >= botmovewiny - topmovewiny + 1) then begin
    gotoxy(1,topmovewiny);
    for i := topmovewiny to movey do
      movelineup(i)
  end;
  if odd(moves) then begin
    gotoxy(1,movey);
    write((moves div 2 + 1):2)
  end;
  gotoxy(4,movey);
  write(algebraic(kfrom),movech[c = empty],algebraic(kto));
  gotoxy(1,wherey+1);
  if wherey <= botmovewiny - 1 then
    movey := wherey
  else
    movey := botmovewiny - 1
end; { showmove }

procedure showmessage(msg : linestring);
{ Display message, up to three 10-character lines, in message window }
{  Message is passed as single linestring  msg  of <= 30 characters }
const
  linelength : byte = 10;
var
  i, j : byte;
begin
  messagewindow;
  messageon := true;
  for i := 1 to 3 do begin
    gotoxy(1,i+21);
    for j := 1 to linelength do
      write(' ');
    gotoxy(1,wherey);
    write(copy(msg, 1 + (i - 1) * linelength, linelength))
  end
end; { showmessage }

procedure clearmessage;
begin
  showmessage('');
  messageon := false
end; { clearmessage }

procedure displaylogo (currentplayer : player);
const
  colorstr  : truthstr = ('Bl','Or');
  playerstr : truthstr = ('Plr','Com');
begin
  gotoxy(2,4);
  write(colorstr[rightsideup]);
  gotoxy(2,6);
  write(colorstr[not rightsideup]);
  gotoxy(6,4);
  write(playerstr[rightsideup xor (currentplayer = orangepc)]);
  gotoxy(6,6);
  write(playerstr[rightsideup xor (currentplayer = blackpc)]);
end; { displaylogo }

procedure initjetan;
{ Initialize jetan global variables }

procedure getvideomode;
var
  reg : regpack;
begin
  with reg do begin
    { Int 10H Fct 0FH  Get Video Mode }
    ax.h := $0F;
    intr($10,Dos.Registers(reg));
    if ax.l = $07 then { Hercules monochrome monitor }
      error(^M^J' CGA Required')
  end
end; { getvideomode }

procedure drawline (lineno : integer);
{ Draw horizontal green line across left of screen at line no }
var
  i : byte;
begin
  sprite := table[greenline];
  workbuffer := colorbuffer;
  for i := 0 to 3 do
    xorspritew(20 * i - 4, 8 * lineno - 3);
  colorbuffer := workbuffer
end; { drawline }

begin { initjetan }
  getvideomode;
  graphcolormode;
  palette(2);
  textcolor(lightgreen);
  randomize;

  rightsideup := true;
  invertingboard := false;
  gotoxy(3,1);
  write('JETAN');
  drawline(3);
  displaylogo(blackpc);
  drawline(8);
  drawline(21);
  showmessage('V2.2 c1993  Silver   Moon Ltd');

  { default lookahead of 2 ply, max movetime of 300 sec = 5 min }
  lookahead := initlookahead;
  level := initlevel;
  movetime := initmovetime[level];

  halfmaxint := maxint div 2;
  changesides := false;
  fullthoatmove := true;
  soundon := true;
  jetanover := false

end; { initjetan }

procedure termjetan;
{ Back to text screen }
begin
  textmode(C80)
end; { termjetan }

procedure playgame;
{ Play one game }
const
  { constants for cursor keys, to move cursor around on board }
  upleftkey =   'G'; upkey =   'H'; uprightkey =   'I';
  leftkey =     'K';                rightkey =     'M';
  downleftkey = 'O'; downkey = 'P'; downrightkey = 'Q';
  fctresponse : charset = [upleftkey,  upkey,  uprightkey,
                           leftkey,            rightkey,
                           downleftkey,downkey,downrightkey];
  CtrlA =  #1;  CtrlC =  #3;  CtrlF =  #6;
  CtrlI =  #9;  CtrlL = #12;  CtrlN = #14;
  CtrlP = #16;  CtrlQ = #17;  CtrlS = #19;
  CtrlT = #20;  CtrlZ = #26;
  response : charset = [CR,CtrlA,CtrlC,CtrlF,CtrlI,CtrlL,CtrlN,
                           CtrlP,CtrlQ,CtrlS,CtrlT,CtrlZ];
var
  mainboard : board;
  list : movelist;
  currentplayer : player;
  initmove : real;
  getfrom, getto : squarenum;
  searchon, quitgame : boolean;

function sqx (var k : squarenum): integer;
{ X coordinate, in pixels, of square on board }
begin
  if rightsideup then
    sqx := leftmargin + ((k - minfile) mod 16) * 24 + 12
  else { board is upside down }
    sqx := leftmargin + ((squarenumsize - k - minfile) mod 16) * 24 + 12
end; { sqx }

function sqy (var k : squarenum): integer;
{ Y coordinate, in pixels, of square on board }
begin
  if rightsideup then
    sqy := ((k - minrank) div 16) * 20 + 10
  else { board is upside down }
    sqy := ((squarenumsize - k - minrank) div 16) * 20 + 10
end; { sqy }

procedure writesquare (var k : squarenum; var c : contents);
{ Write contents of square onto board }
begin
  if odd(k div 16 + k mod 16) then { orange square }
    case c of
      empty     : sprite := table[emptyo];
      othoat    : sprite := table[othoato];
      opanthan  : sprite := table[opanthano];
      owarrior  : sprite := table[owarrioro];
      opadwar   : sprite := table[opadwaro];
      odwar     : sprite := table[odwaro];
      oflier    : sprite := table[ofliero];
      oprincess : sprite := table[oprincesso];
      ochief    : sprite := table[ochiefo];
      bthoat    : sprite := table[bthoato];
      bpanthan  : sprite := table[bpanthano];
      bwarrior  : sprite := table[bwarrioro];
      bpadwar   : sprite := table[bpadwaro];
      bdwar     : sprite := table[bdwaro];
      bflier    : sprite := table[bfliero];
      bprincess : sprite := table[bprincesso];
      bchief    : sprite := table[bchiefo]
    end
  else { black square }
    case c of
      empty     : sprite := table[emptyb];
      othoat    : sprite := table[othoatb];
      opanthan  : sprite := table[opanthanb];
      owarrior  : sprite := table[owarriorb];
      opadwar   : sprite := table[opadwarb];
      odwar     : sprite := table[odwarb];
      oflier    : sprite := table[oflierb];
      oprincess : sprite := table[oprincessb];
      ochief    : sprite := table[ochiefb];
      bthoat    : sprite := table[bthoatb];
      bpanthan  : sprite := table[bpanthanb];
      bwarrior  : sprite := table[bwarriorb];
      bpadwar   : sprite := table[bpadwarb];
      bdwar     : sprite := table[bdwarb];
      bflier    : sprite := table[bflierb];
      bprincess : sprite := table[bprincessb];
      bchief    : sprite := table[bchiefb]
    end;
  putspritew(sqx(k),sqy(k))
end; { writesquare }

procedure displaysquare (var k : squarenum; var c : contents);
{ Display a single square on screen }
begin
  workbuffer := colorbuffer;
  writesquare(k,c);
  colorbuffer := workbuffer
end; { displaysquare }

procedure displaywait (k : squarenum; c : contents);
{ Display a single square on screen, then wait }
begin
  displaysquare(k,c);
  delay(200)
end; { displaywait }

procedure displaymove (var kfrom, kto : squarenum; var c : contents);
{ Display a move on screen }
var
  i, j : integer;
begin
  with mainboard do begin
    displaywait(kfrom,empty);
    displaywait(kfrom,sq[kfrom]);
    displaywait(kfrom,empty);
    displaywait(kto,sq[kfrom]);
    displaywait(kto,empty);
    displaysquare(kto,sq[kfrom])
  end
end; { displaymove }

procedure writecursor (var cursor : squarenum);
{ Write cursor onto board }
begin
  if odd(cursor div 16 + cursor mod 16) then { orange square }
    sprite := table[cursoro]
  else { black square }
    sprite := table[cursorb];
  xorspritew(sqx(cursor),sqy(cursor))
end; { writecursor }

procedure displaycursor (var cursor : squarenum);
{ Display cursor on screen }
begin
  with mainboard do begin
    workbuffer := colorbuffer;
    writecursor(cursor);
    colorbuffer := workbuffer
  end;
end; { displaycursor }

procedure movecursor (var cursor : squarenum;
                      ch : char; manchosen : squarenum);
{ Move cursor to updated location on board }

function newcursor (var cursor : squarenum; var ch : char): squarenum;
begin
  if rightsideup then
    case ch of
      upleftkey    : newcursor := cursor + up1left1;
      upkey        : newcursor := cursor + up1;
      uprightkey   : newcursor := cursor + up1right1;
      leftkey      : newcursor := cursor + left1;
      rightkey     : newcursor := cursor + right1;
      downleftkey  : newcursor := cursor + down1left1;
      downkey      : newcursor := cursor + down1;
      downrightkey : newcursor := cursor + down1right1
    end
  else { board is upside down }
    case ch of
        upleftkey    : newcursor := cursor + down1right1;
        upkey        : newcursor := cursor + down1;
        uprightkey   : newcursor := cursor + down1left1;
        leftkey      : newcursor := cursor + right1;
        rightkey     : newcursor := cursor + left1;
        downleftkey  : newcursor := cursor + up1right1;
        downkey      : newcursor := cursor + up1;
        downrightkey : newcursor := cursor + up1left1
      end
end; { newcursor }

begin { movecursor }
  with mainboard do
    if sq[newcursor(cursor,ch)] <> border then begin
      displaysquare(cursor,sq[cursor]);
      cursor := newcursor(cursor,ch);
      displaycursor(cursor);
      if manchosen > 0 then begin
        workbuffer := colorbuffer;
        writesquare(manchosen,sq[manchosen]);
        writecursor(manchosen);
        colorbuffer := workbuffer
      end
    end
end; { movecursor }

procedure displayboard;
{ Display entire board on screen }
var
  i, j, k : squarenum;
begin
  workbuffer := colorbuffer;
  for i := 0 to 9 do
    for j := 0 to 9 do begin
      k := minrank + 16 * i + minfile + j;
      writesquare(k,mainboard.sq[k])
    end;
  colorbuffer := workbuffer
end; { displayboard }

function other (pl : player): player;
{ Return other player's color }
begin
  if pl = orangepc then
    other := blackpc
  else { pl = blackpc }
    other := orangepc
end; { other }

procedure exitgame (var ch :char);
{ New game or quit game }
var
  yn : char;
begin
  case ch of
    CtrlN : showmessage('New game?   (Y/N)');
    CtrlQ : showmessage('Quit game?  (Y/N)')
  end;
  yn := getkey(yn,['Y','N',ESC],functionkey);
  if (yn = 'Y') and (not functionkey) then begin
    quitgame := true;
    case ch of
      CtrlN : gameover := true;
      CtrlQ : jetanover := true
    end
  end;
  if (ch = CtrlN) or (not quitgame) then
    clearmessage;
  if functionkey then
    beep(440,250)
end; { exitgame }

function central (var bd: board; var k : squarenum): integer;
{ Check how close piece is to center of board }
var
  temp : integer;
  kmod, kdiv : byte;
begin
  with bd do begin
    { file of square k }
    kmod := k mod 16;
    { rank of square k }
    kdiv := k div 16;
    { test for rank or file; "else" renders test zones concentric }
    if (bdgauge[kmod] = 1) or (bdgauge[kdiv] = 1) then
      temp := 5
    else if (bdgauge[kmod] = 2) or (bdgauge[kdiv] = 2) then
      temp := 10
    else if (bdgauge[kmod] = 3) or (bdgauge[kdiv] = 3) then
      temp := 15
    else if (bdgauge[kmod] = 4) or (bdgauge[kdiv] = 4) then
      temp := 18
    else { (bdgauge[kmod] = 5) or (bdgauge[kdiv] = 5) }
      temp := 20;
    case sq[k] of
      opanthan,  bpanthan,
      odwar,     bdwar     : temp := 2 * temp;
      oprincess, bprincess : temp := 100 - 100 * temp
    end;
    central := temp
  end
end; { central }

procedure checkchessmen (kfrom, kto : squarenum; pl : player; var bd : board);
{ Check and update variables dependent upon piece movement and location }
var
  opp : player;
  i : integer;
  sqkfrom, sqkto : contents;
begin
  with bd do begin
    opp := other(pl);
    sqkfrom := sq[kfrom];
    sqkto := sq[kto];

    { check if move is a capture }
    if ((othoat <= sqkfrom) and (sqkfrom <= ochief) and
        (bthoat <= sqkto) and (sqkto <= bchief)) or
       ((bthoat <= sqkfrom) and (sqkfrom <= bchief) and
        (othoat <= sqkto) and (sqkto <= ochief)) then
    begin
      { if captured piece is chief, record captor }
      if sqkto in [ochief,bchief] then
        chiefcaptor := sqkfrom
      { if captured piece is princess, record captor }
      else if sqkto in [oprincess,bprincess] then
        princesscaptor := sqkfrom;
      { remove captured piece from list }
      chessmen[opp,0] := kto;
      i := numchessmen[opp];
      while chessmen[opp,i] <> kto do
        i := i - 1;
      while i < numchessmen[opp] do begin
        chessmen[opp,i] := chessmen[opp,i + 1];
        i := i + 1
      end;
      numchessmen[opp] := numchessmen[opp] - 1
    end

    { check if move is a princess "escape" move }
    else if sqkfrom in [oprincess,bprincess] then
      if not escapeyet[pl] then
        if (abs(kfrom div 16 - kto div 16) > 3) or
           (abs(kfrom mod 16 - kto mod 16) > 3) then
          escapeyet[pl] := true;

    { update array recording location of pieces }
    i := numchessmen[pl];
    chessmen[pl,0] := kfrom;
    while chessmen[pl,i] <> kfrom do
      i := i - 1;
    chessmen[pl,i] := kto
  end
end; { checkchessmen }

procedure initgame;
{ Initialize game variables }
var
  i, j : integer;
begin
  with mainboard do begin

    { Define border spaces around board, empty spaces on board }
    for i := 0 to 2 do
      for j := 0 to 15 do
        sq[16*i + j] := border;
    for i := 3 to 12 do begin
      for j := 0 to 2 do
        sq[16*i + j] := border;
      for j := 3 to 12 do
        sq[16*i + j] := empty;
      for j := 13 to 15 do
        sq[16*i + j] := border
    end;
    for i := 13 to 15 do
      for j := 0 to 15 do
        sq[16*i + j] := border;

    { Set up pieces in initial position }
    sq[51] := owarrior; sq[52] := opadwar; sq[53] := odwar; sq[54] := oflier;
      sq[55] := oprincess; sq[56] := ochief;
    sq[57] := oflier; sq[58] := odwar; sq[59] := opadwar; sq[60] := owarrior;
    sq[67] := othoat; sq[76] := othoat;
    for i := 68 to 75 do
      sq[i] := opanthan;
    for i := 180 to 187 do
      sq[i] := bpanthan;
    sq[179] := bthoat; sq[188] := bthoat;
    sq[195] := bwarrior; sq[196] := bpadwar; sq[197] := bdwar; sq[198] := bflier;
      sq[199] := bchief; sq[200] := bprincess;
    sq[201] := bflier; sq[202] := bdwar; sq[203] := bpadwar; sq[204] := bwarrior;

    { Initialize display for new game }
    rightsideup := true;
    displayboard;
    movewindow;
    for i := 1 to (botmovewiny - topmovewiny + 1) do begin
      gotoxy(1,topmovewiny+i-1);
      for j := 1 to 10 do
        write(' ')
    end;

    { Record all initial positions of pieces from typed constant array;
      since move lists are drawn up in the order given in this array,
      this ordering is to maximize speed of game tree search }
    chessmen := initchessmen;
    numchessmen[orangepc] := maxpieces;
    numchessmen[blackpc] := maxpieces;

    { Initialize list of possible moves to zero }
    possible.nmoves := 0;
    for i := 0 to maxmoves do begin
      possible.movefrom[i] := 0;
      possible.moveto[i] := 0;
      possible.key[i] := 0
    end;
    gamelist.nmoves := 0;

    { Neither player's princess has yet made 10-space escape move }
    escapeyet[orangepc] := false;
    escapeyet[blackpc] := false;

    { Neither player's chief or princess has yet been captured }
    chiefcaptor := empty;
    princesscaptor := empty;

    { Game not yet drawn in 10 move, with 5 pieces or less to each player }
    drawin10 := 0;

    { Computer is human player's opponent }
    human := blackpc;
    computer := other(human);
    currentplayer := human;
    displaylogo(currentplayer);

    { Set and show chess clock }
    inittime := 0;
    inittime := gettime;
    chessclock[human] := 0;
    chessclock[computer] := 0;
    split[human] := gettime;
    split[computer] := gettime;
    showclock(computer);
    showclock(human);

    moves := 0;
    movey := topmovewiny;
    quitgame := false;
    gameover := false;
    opening := true;
    blinker := true
  end
end; { initgame }

procedure termgame;
{ After game over, wait for response }
begin
  while not quitgame do begin
    ch := getkey(ch,[CtrlN,CtrlQ,ESC],functionkey);
    if not functionkey then
      exitgame(ch)
  end
end; { termgame }

procedure addmove (var legal : movelist; kfrom : squarenum; kto : squarenum);
{ Add legal move to move list }
begin
  with legal do begin
    nmoves := nmoves + 1;
    movefrom[nmoves] := kfrom;
    moveto[nmoves] := kto
  end
end; { addmove }

function makelist (var legal : movelist; pl : player; var bd : board): integer;
{ Make list of legal moves, return number of legal moves }
var
  i : integer;

function target (k : squarenum; var bd : board; pl : player) : boolean;
{ Square is legal target, if empty or piece of opposite color }
var
  c : contents;
begin
  with bd do
    c := sq[k];
    if pl = orangepc then
      target := (c = empty) or ((bthoat <= c) and (c <= bchief))
    else { pl = blackpc }
      target := (c = empty) or ((othoat <= c) and (c <= ochief))
end; { target }

{ include procedure getlegalmoves -- }
{ contains definition of legal moves for pieces: }
{$I JETAN1.P }

begin { makelist }
  legal.nmoves := 0;
  with bd do begin
    i := 1;
    while i <= numchessmen[pl] do begin
      getlegalmoves(chessmen[pl,i],bd,pl,legal);
      i := i + 1
    end
  end;
  makelist := legal.nmoves
end; { makelist }

function chiefnum (var ok : boolean; pl : player; var bd : board) : integer;
var
  j : integer;
begin
  with bd do begin
    j := maxchief;
    while (j > 0) and not(sq[chessmen[pl,j]] in [ochief,bchief]) do
      j := j - 1;
    ok := j > 0;
    chiefnum := chessmen[pl,j]
  end
end; { chiefnum }

function chvsch (var list : movelist; pl : player; var bd : board) : boolean;
{ Find whether chief is in check }
var
  ok : boolean;
  i, j : integer;
  chief, vschief : squarenum;

begin
  with list do begin
    i := nmoves;
    with bd do begin
      ok := true;
      chief := chiefnum(ok,pl,bd);
      vschief := chiefnum(ok,other(pl),bd)
    end;
    if ok then begin
      moveto[0] := chief;
      movefrom[0] := vschief;
      while [movefrom[i],moveto[i]] <> [chief,vschief] do
        i := i - 1;
      chvsch := i > 0
    end
    else { a chief has already been taken }
      chvsch := true
{    chvsch := (i > 0) or not (bd.sq[chief] in [ochief,bchief]) }
  end
end; { chvsch }

function incheck (var list : movelist; pl : player; var bd : board) : boolean;
{ Find whether princess is in check }
var
  i : integer;
  princess : squarenum;
begin
  with list do begin
    i := nmoves;
    with bd do
      princess := chessmen[pl,numchessmen[pl]];
    if bd.sq[princess] in [oprincess,bprincess] then begin
      moveto[0] := princess;
      while moveto[i] <> princess do
        i := i - 1;
      incheck := i > 0
    end
    else { princess has already been taken }
      incheck := true
{    incheck := (i > 0) or not (bd.sq[princess] in [oprincess,bprincess]) }
  end
end; { incheck }

procedure getmove (var list : movelist; pl : player;
                   var getfrom, getto : squarenum);
{ Get current player's move }
var
  i : integer;
  princess : squarenum;

procedure gethuman (var list : movelist; var getfrom, getto : squarenum);
{ Get human's move }
var
  i : integer;
  cursor, manchosen : squarenum;
  movemade, movelegal, targetchosen : boolean;
type
  initcursortype = array [player] of squarenum;
const
  initcursor : initcursortype = (76,179);

procedure processkey (var ch : char);
{ Process keyboard input from human player }
begin
  with mainboard do
    case ch of
      CR : if manchosen > 0 then begin
             if (sq[cursor] = empty) or
                (((sq[manchosen] in [othoat..ochief]) and
                  (sq[cursor] in [bthoat..bchief])) or
                 ((sq[manchosen] in [bthoat..bchief]) and
                  (sq[cursor] in [othoat..ochief]))) then
             begin
               i := 1;
               while (i <= list.nmoves) and (not movelegal) do
                 if (list.movefrom[i] = manchosen) and
                    (list.moveto[i] = cursor) then
                   movelegal := true
                 else
                   i := i + 1;
               if (sq[manchosen] in [oprincess,bprincess]) and
                  not (movelegal or escapeyet[pl]) then
               begin
                 escapeyet[pl] := true;
                 movelegal := true;
                 i := 1;
                 list.movefrom[i] := manchosen;
                 list.moveto[i] := cursor;
                 showmessage('  Escape')
               end;
               movemade := movelegal;
               if not movemade then
                 beep(440,250)
             end
             else if manchosen = cursor then
               manchosen := 0
           end
           else begin
             if ((human = blackpc) and
                 (sq[cursor] in [bthoat..bchief])) or
                ((human = orangepc) and
                 (sq[cursor] in [othoat..ochief])) then
               manchosen := cursor
             else
               beep(440,250)
           end;
      CtrlC : begin { change color with the move }
                showmessage('  Change    color     w/move');
                displaysquare(cursor,sq[cursor]);
                computer := other(computer);
                human := other(human);
                movemade := true;
                changesides := true;
                displaylogo(other(currentplayer))
              end;
      CtrlF : begin { offer computer first move }
                { NB -- this feature misfunctions at present--
                  when computer accepts first move, it has a disconcerting
                  tendency to move its princess, which it shouldn't do
                  if it is calling the quickopening routine }
                if moves = 0 then begin
                  if random < 0.5 then begin
                    movemade := true;
                    changesides := true;
                    pl := other(pl);
                    displaysquare(cursor,sq[cursor]);
                    showmessage(' Computer   moves     first')
                  end
                  else
                    showmessage('   You       move      first')
                end
              end;
      CtrlI : begin { invert board }
                showmessage('  Invert    board');
                rightsideup := not rightsideup;
                invertingboard := true;
                displayboard;
                displaycursor(cursor);
                displaylogo(currentplayer);
                showclock(human);
                showclock(computer);
                invertingboard := false;
                clearmessage
              end;
      CtrlL : begin { set game level }
                showmessage('Level =     (+/-)');
                messagewindow;
                repeat
                  gotoxy(9,22);
                  write(level:1);
                  ch := getkey(ch,['+','-',CR,ESC],functionkey);
                  messagewindow;
                  if not functionkey then begin
                    case ch of
                      '+' : if level < maxlevel then
                              level := level + 1;
                      '-' : if level > minlevel then
                              level := level - 1;
                    end;
                    movetime := initmovetime[level]
                  end
                until ch in [CR,ESC];
                clearmessage;
                beep(880,100)
              end;
      CtrlN,
      CtrlQ : exitgame(ch); { new game or quit game }
      CtrlS : begin { change sides with computer }
                showmessage('  Change    sides');
                displaysquare(cursor,sq[cursor]);
                computer := other(computer);
                human := other(human);
                currentplayer := other(currentplayer);
                movemade := true;
                changesides := true;
                displaylogo(currentplayer);
                clearmessage
              end;
      CtrlT : begin { toggle full thoat move }
                fullthoatmove := not fullthoatmove;
                tempbd := mainboard;
                i := makelist(list,human,tempbd);
                if fullthoatmove then
                  showmessage('Full thoat move on')
                else
                  showmessage('Full thoat move off')
              end;
      CtrlZ : begin { toggle sound on/off }
                soundon := not soundon;
                if soundon then
                  showmessage('Sound on')
                else
                  showmessage('Sound off')
              end;
      else
        beep(440,250)
    end
end; { processkey }

begin { gethuman }
  movemade := false;
  movelegal := false;
  targetchosen := false;
  manchosen := 0;
  cursor := initcursor[currentplayer];
  displaycursor(cursor);
  while not (movemade or quitgame) do begin
    ch := getkey(ch,response + fctresponse,functionkey);
    if messageon then
      clearmessage;
    if functionkey then begin
      if ch in fctresponse then
        movecursor(cursor,ch,manchosen)
      else
        beep(440,250)
    end
    else
      processkey(ch)
  end;
  getfrom := list.movefrom[i];
  getto := list.moveto[i]
end; { gethuman }

procedure getcomputer (var list : movelist; var getfrom, getto : squarenum);
{ Get computer's move }
var
  max, i, ix, oldix, j : integer;
  bestfrom, bestto, k, princess : squarenum;
  ok, princessincheck : boolean;

function eval (var bd: board; pl : player): integer;
{ Evaluate board pattern }
var
  opp : player;
  i, score : integer;
  princess, chief : squarenum;
  c, plc : contents;
begin
  with bd do begin
    if chiefcaptor <> empty then begin { a chief has been captured }
      if ((human = orangepc) and (chiefcaptor = ochief)) or
         ((human = blackpc) and (chiefcaptor = bchief))
      then
        score := -maxint { human's chief has captured computer's chief }
      else if ((computer = orangepc) and (chiefcaptor = ochief)) or
              ((computer = blackpc) and (chiefcaptor = bchief))
      then
        score := maxint { computer's chief has captured human's chief }
      else { chiefcaptor in [othoat..oflier,bthoat..bflier] }
        score := 0 { game is drawn, chief taken by other than chief }
    end
    else if princesscaptor <> empty then begin { a princess has been captured }
      if ((human = orangepc) and (princesscaptor in [othoat..ochief])) or
         ((human = blackpc) and (princesscaptor in [bthoat..bchief]))
      then
        score := -maxint { human has captured computer's princess }
      else if ((computer = orangepc) and (princesscaptor in [othoat..ochief])) or
              ((computer = blackpc) and (princesscaptor in [bthoat..bchief]))
      then
        score := maxint { computer has captured human's princess }
    end
    else begin { neither player wiped out }
      tempbd := bd;
      opp := other(pl);

      { basic score is (# of player's moves) - (# of opp's moves) }
      score := possible.nmoves - makelist(templist,opp,tempbd);

      { adjustment for position of pieces }
      i := numchessmen[human];
      while i > 0 do begin
        score := score
               - value[sq[chessmen[human,i]]]
               - central(bd,chessmen[human,i]);
        i := i - 1
      end;
      i := numchessmen[computer];
      while i > 0 do begin
        score := score
               + value[sq[chessmen[computer,i]]]
               + central(bd,chessmen[computer,i]);
        i := i - 1
      end;

      { find whether player's chief or princess is threatened;    }
      { these routines depend on assignment of princess to        }
      { chessmen[pl,numchessmen[pl]] and chief to no later than   }
      { chessmen[pl,5], and must be revised if these assignments  }
      { are altered                                               }

      { find whether player's chief is threatened }
      if pl = orangepc then
        plc := ochief
      else
        plc := bchief;

      { find player's chief }
      i := maxchief;
      c := sq[chessmen[pl,i]];
      while (c <> plc) and (i > 1) do begin
        i := i - 1;
        c := sq[chessmen[pl,i]]
      end;
      chief := chessmen[pl,i];

      { find whether player's chief is targeted on opp's move list }
      with templist do begin
        i := nmoves;
        moveto[0] := chief;
        repeat
          while moveto[i] <> chief do
            i := i - 1;
          if i > 0 then begin
            c := sq[movefrom[i]];
            if ((othoat <= c) and (c <= oflier)) or
               ((bthoat <= c) and (c <= bflier)) then { threatened draw }
              score := -score
            { else sq[movefrom[i]] in [ochief,bchief] }
            else if opp = human then { computer's chief in check }
              score := score - halfmaxint
            else { opp = computer } { human's chief in check }
              score := score + halfmaxint;
            i := i - 1
          end
        until (i <= 0) or (abs(score) = maxint)
      end;

      { find whether player's princess is threatened }
      princess := chessmen[pl,numchessmen[pl]];

      { find whether player's princess targeted on opp's move list }
      with templist do begin
        i := nmoves;
        moveto[0] := princess;
        while moveto[i] <> princess do
          i := i - 1
      end;
      if i > 0 then begin
        if opp = human then { computer's princess in check }
          score := score - halfmaxint
        else { opp = computer } { human's princess in check }
          score := score + halfmaxint
      end
    end;

    eval := score
  end
end; { eval }

procedure quicksort (var list : movelist; m, n : integer);
{ Sort move list for quicker game tree search }
var
  i, j, k, kkey : integer;
  kfrom, kto : squarenum;
begin
  if m < n then begin
    with list do begin
      i := m;
      j := n + 1;
      k := key[m];
      repeat
        repeat
          i := i + 1
        until key[i] <= k;
        repeat
          j := j - 1
        until key[j] >= k;
        if i < j then begin
          kfrom := movefrom[j];
          kto := moveto[j];
          kkey := key[j];
          movefrom[j] := movefrom[i];
          moveto[j] := moveto[i];
          key[j] := key[i];
          movefrom[i] := kfrom;
          moveto[i] := kto;
          key[i] := kkey
        end
      until i >= j;
      kfrom := movefrom[m];
      kto := moveto[m];
      kkey := key[m];
      movefrom[m] := movefrom[j];
      moveto[m] := moveto[j];
      key[m] := key[j];
      movefrom[j] := kfrom;
      moveto[j] := kto;
      key[j] := kkey;
      quicksort(list,m,j-1);
      quicksort(list,j+1,n)
    end
  end
end; { quicksort }

procedure trymove (tryfrom, tryto : squarenum; pl : player; var bd : board);
{ Try move, updating board }
begin
  with bd do begin
    checkchessmen(tryfrom,tryto,pl,bd);
    sq[tryto] := sq[tryfrom];
    sq[tryfrom] := empty
  end
end; { trymove }

function findmin (look : integer; var list : movelist; var bd : board;
                  cutoff : integer; var bestfrom, bestto : squarenum): integer;
                  forward;

function findmax (look : integer; var list : movelist; var bd : board;
                  cutoff : integer; var initix : integer;
                  var bestfrom, bestto : squarenum): integer;
{ Get computer's best move }
var
  newlist : movelist;
  newbd : board;
  i, maxscore, score, nm : integer;
  junkfrom, junkto : squarenum;
begin
  with list do begin
    gotoxy(5,2);
    write(blinkch[blinker]);
    blinker := not blinker;
    for i := 1 to nmoves do
      key[i] := central(bd,moveto[i]);
    quicksort(list,1,nmoves);
    if nmoves > 0 then begin
      maxscore := -maxint;
      i := initix;
      bestfrom := movefrom[i];
      bestto := moveto[i];
      if searchon then
        repeat
          newbd := bd;
          trymove(movefrom[i],moveto[i],computer,newbd);
          if look <= 1 then begin
            if look <= 0 then
              score := eval(newbd,human)
            else begin
              nm := makelist(newlist,human,newbd);
              score := findmin(look-1,newlist,newbd,maxscore,junkfrom,junkto)
            end
          end
          else begin
            nm := makelist(newlist,human,newbd);
            score := findmin(look-1,newlist,newbd,maxscore,junkfrom,junkto)
          end;
          if score > maxscore then begin
            maxscore := score;
            bestfrom := movefrom[i];
            bestto := moveto[i]
          end;
          i := i + 1
        until (i > nmoves) or (maxscore >= cutoff)
    end
    else { no legal move }
      maxscore := eval(bd,human)
  end;
  searchon := gettime - initmove < movetime;
  initix := i;
  findmax := maxscore
end; { findmax }

function findmin;
{ Get human's best move }
var
  newlist : movelist;
  newbd : board;
  i, initix, minscore, score, nm : integer;
  junkfrom, junkto : squarenum;
begin
  with list do begin
    gotoxy(5,2);
    write(blinkch[blinker]);
    blinker := not blinker;
    for i := 1 to nmoves do
      key[i] := central(bd,moveto[i]);
    quicksort(list,1,nmoves);
    if nmoves > 0 then begin
      minscore := maxint;
      initix := 1;
      i := 1;
      bestfrom := movefrom[1];
      bestto := moveto[1];
      if searchon then
        repeat
          newbd := bd;
          trymove(movefrom[i],moveto[i],human,newbd);
          if look <= 1 then begin
            if look <= 0 then
              score := eval(newbd,computer)
            else begin
              nm := makelist(newlist,computer,newbd);
              score := findmax(look-1,newlist,newbd,minscore,initix,junkfrom,junkto)
            end
          end
          else begin
            nm := makelist(newlist,computer,newbd);
            score := findmax(look-1,newlist,newbd,minscore,initix,junkfrom,junkto)
          end;
          if score < minscore then begin
            minscore := score;
            bestfrom := movefrom[i];
            bestto := moveto[i]
          end;
          i := i + 1
        until (i > nmoves) or (minscore <= cutoff)
    end
    else { no legal move }
      minscore := eval(bd,computer)
  end;
  searchon := gettime - initmove < movetime;
  findmin := minscore
end; { findmin }

procedure quickopening (var list : movelist; var bestfrom, bestto : squarenum);
{ Provides computer with quicker move in opening game }
{  In absence of a body of work on openings, this routine }
{  provides the equivalent of a library of opening moves }
const
  maxnmoves = 36;
  imprecision = 5;
const
  winnow : truthbyte = (6,2);
var
  i : integer;
  k : squarenum;
begin
  { series of sorts with most significant sort key last }
  with list do begin

    { select moves to closer to center of board }
    for i := 1 to nmoves do
      key[i] := central(mainboard,moveto[i]);
    quicksort(list,1,nmoves);

    { select moves by pieces still in back two rows }
    for i := 1 to nmoves do
      key[i] := ord((movefrom[i] div 16) in [3,4,11,12]);
    quicksort(list,1,nmoves);

    { omit moves by princess }
    with mainboard do
      k := chessmen[pl,numchessmen[pl]];
    movefrom[0] := k;
    i := nmoves;
    while i > 0 do begin
      while movefrom[i] <> k do
        i := i - 1;
      if i > 0 then begin
        while i < nmoves do begin
          movefrom[i] := movefrom[i + 1];
          moveto[i] := moveto[i + 1];
          i := i + 1
        end;
        nmoves := nmoves - 1
      end
    end;

    { truncate list of possible moves }
    if nmoves >= maxnmoves + imprecision then
      nmoves := maxnmoves + random(imprecision);

    { choose half of these at random to examine further }
    for i := 1 to nmoves do
      key[i] := random(2);
    quicksort(list,1,nmoves);
    nmoves := nmoves div winnow[moves >= 2]; { first move, choose one in six }

    { final sort: select moves to closer to center of board }
    for i := 1 to nmoves do
      key[i] := central(mainboard,moveto[i]);
    quicksort(list,1,nmoves);

    { search remaining game tree, 2-ply lookahead }
    i := 1;
    max := findmax(2,list,mainboard,maxint,i,bestfrom,bestto)
  end
end; { quickopening }

procedure shuffle (var list : movelist);
{ Shuffles move list subranges of moves by same rank of piece }
{  Keeps program from favoring one piece over another of same kind }
{  Also provides random element without sacrificing move quality }
var
  i, m, n : squarenum;
begin
  with list do begin
    n := nmoves;
    m := n;
    movefrom[0] := 0;
    with mainboard do
      while n > 0 do begin
        while sq[movefrom[m]] = sq[movefrom[n]] do
          m := m - 1;
        for i := n downto m + 1 do
          key[i] := random(100);
        quicksort(list,m+1,n);
        n := m
      end
  end
end; { shuffle }

procedure deletechecks(var list : movelist; pl : player; var bd : board);
{ Eliminate from move list those moves which do not deliver from check }
var
  i, n : integer;
  tempbd : board;
begin
  with list do begin
    for i := 1 to nmoves do begin
      tempbd := bd;
      trymove(movefrom[i],moveto[i],pl,tempbd);
      n := makelist(templist,other(pl),tempbd);
      key[i] := ord(not (incheck(templist,pl,tempbd) or
                          chvsch(templist,pl,tempbd)))
    end;
    quicksort(list,1,nmoves);
    key[0] := 1;
    i := nmoves;
    while key[i] = 0 do
      i := i - 1;
    nmoves := i
  end
end; { deletechecks }

procedure desperatestraits;
{ computer is in check and can't get out by ordinary means }
var
  i, j, k : integer;
  enemychief : squarenum;
begin
  with mainboard do
    if not escapeyet[computer] then begin
      { princess will escape }
      escapeyet[computer] := true;
      princess := chessmen[computer,numchessmen[computer]];
      list.nmoves := 0;
      showmessage('  Escape');
      for i := 0 to 9 do
        for j := 0 to 9 do begin
          k := minrank + 16 * i + minfile + j;
          if (sq[k] = empty) then
            if (abs(k div 16 - princess div 16) > 3) and
               (abs(k mod 16 - princess mod 16) > 3) then
              addmove(list,princess,k)
        end;
      shuffle(list);
      getcomputer(list,bestfrom,bestto)
    end
    else begin
      { settle for draw if possible }
      showmessage(' Princess  trapped');
      tempbd := mainboard;
      i := makelist(templist,computer,tempbd);
      with templist do begin
        i := nmoves;
        enemychief := chiefnum(ok,other(pl),mainboard);
        moveto[0] := enemychief;
        while moveto[i] <> enemychief do
          i := i - 1;
        if i > 0 then begin
          movefrom[1] := movefrom[i];
          moveto[1] := moveto[i];
          nmoves := 1;
          getcomputer(templist,bestfrom,bestto)
        end
      end
    end
end; { desperatestraits }

begin { getcomputer }
  { if computer in check, release it from time constraints }
  if chvsch(list,computer,mainboard) then
    with list do begin
      movefrom[0] := byte(chiefnum(ok,pl,mainboard));
      moveto[0] := byte(chiefnum(ok,other(pl),mainboard));
      ix := nmoves;
      with mainboard do
        while ([sq[movefrom[ix]],sq[moveto[ix]]] <> [ochief,bchief]) do
          ix := ix - 1;
      if ix > 0 then begin
        movefrom[1] := movefrom[ix];
        moveto[1] := moveto[ix];
        nmoves := 1
      end
    end
  else { temp } if incheck(list,computer,mainboard) then begin
(*    initmove := initmove + maxint; *) { temp }
    deletechecks(list,computer,mainboard)
  end;
  searchon := true;
  if list.nmoves = 0 then begin { no legal move }
    quitgame := true;
    beep(800,250);
    showmessage('Game drawn--no moves')
  end
  else if list.nmoves = 1 then begin { only one legal move }
    getfrom := list.movefrom[1];
    getto := list.moveto[1]
  end
  else begin { list.nmoves > 1 }
    if opening then { in opening game }
      quickopening(list,bestfrom,bestto)
    else begin
      shuffle(list);
      ix := 1;
      repeat
        oldix := ix;
        max := findmax(lookahead,list,mainboard,maxint,ix,bestfrom,bestto);
        { reject move that leaves computer in check }
        tempbd := mainboard;
        trymove(bestfrom,bestto,computer,tempbd);
        i := makelist(templist,human,tempbd);
        princessincheck := incheck(templist,computer,tempbd) or
                            chvsch(templist,computer,tempbd);
        if princessincheck then begin
(*          initmove := initmove + initmovetime[level + 1]; *) { temp }
          if ix = oldix then
            ix := ix + 1
        end
      until (not princessincheck) or (ix >= list.nmoves);
      if princessincheck and (ix >= list.nmoves) then
        desperatestraits
    end;
    getfrom := bestfrom;
    getto := bestto
  end;
  showclock(computer);
  beep(220,500)
end; { getcomputer }

begin { getmove }
  split[pl] := gettime;
  initmove := split[pl];
  if pl = computer then
    getcomputer(list,getfrom,getto)
  else
    gethuman(list,getfrom,getto);

  { test for end of opening game-- }
  { opening also ends if princess in danger; }
  { see procedure makemove }
  if opening then
    with mainboard do
      if (sq[getto] <> empty) or              { capture }
         (sq[getfrom] in [ochief,bchief]) or  { move by chief }
         ((currentplayer = orangepc) and      { center of board crossed }
          (getto div 16 >= 8)) or
         ((currentplayer = blackpc) and
          (getto div 16 <= 7)) or
         (moves >= 16) then                   { ninth move }
        opening := false
end; { getmove }

procedure makemove (kfrom, kto : squarenum; pl : player);
{ Make move on main board }
var
  i : integer;
  k : squarenum;
begin
  with mainboard do begin
    moves := moves + 1;
    displaymove(kfrom,kto,sq[kfrom]);
    checkchessmen(kfrom,kto,currentplayer,mainboard);
    showmove(kfrom,kto,sq[kto],pl);
    with gamelist do begin
      nmoves := nmoves + 1;
      movefrom[nmoves] := kfrom;
      moveto[nmoves] := kto;
      key[nmoves] := ord(sq[kfrom]) shl 8 + ord(sq[kto])
    end;
    sq[kto] := sq[kfrom];
    sq[kfrom] := empty;

    { test whether game is won or drawn, or princess is in danger }
    if (chiefcaptor <> empty) or (princesscaptor <> empty) then begin
      for i := 1 to 5 do begin
        beep(880,100);
        delay(100)
      end;
      if princesscaptor <> empty then
        showmessage(' Princess  captured')
      else if chiefcaptor in [ochief,bchief] then
        showmessage('  Chief    captures   chief')
      else { chiefcaptor in [othoat..oflier,bthoat..bflier] }
        showmessage('  Chief   captured--Game drawn');
      gameover := true
    end
    else if (numchessmen[human] <= insufficientforce) and
            (numchessmen[computer] <= insufficientforce) then begin
      drawin10 := drawin10 + 1;
      if drawin10 > drawmovelimit then begin
        beep(880,500);
        showmessage('Game drawn --forces   too low');
        gameover := true
      end
    end
    else { test for princess in danger }
      with list do begin
        tempbd := mainboard;
        i := makelist(list,pl,tempbd);
        if incheck(list,other(pl),mainboard) then begin
          beep(880,250);
          showmessage(' Princess     in      danger');
          opening := false
        end
        else
          clearmessage
      end
  end
end; { makemove }

begin { playgame }
  initgame;
  repeat
    if makelist(list,currentplayer,mainboard) > 0 then begin
      getmove(list,currentplayer,getfrom,getto);
      if quitgame then
        gameover := true
      else if changesides then
        changesides := false
      else
        makemove(getfrom,getto,currentplayer)
    end
    else begin { player not able to move }
      beep(880,250);
      gameover := true;
      showmessage('Game drawn--no moves')
    end;
    currentplayer := other(currentplayer)
  until gameover;
  termgame
end; { playgame }


begin { main }
  initjetan;
  repeat
    playgame
  until jetanover;
  termjetan
end. { jetan }
