{ --------------------------------------------------------------------------- }
{                       The Game of JETAN -- Version 2.2                      }
{                            Copyright (c)1992 by                             }
{                          Silver Moon Software, Ltd.                         }
{                                                                             }
{    This file contains a revision of Donald Pavia's Turbo Sprite routines    }
{  to provide the board and chessmen graphics for the game of Jetan. Since    }
{  these routines write to a buffer which is copied directly to video RAM,    }
{  the standard Turbo Pascal graphics library GRAPH.P is not needed.          }
{                                                                             }
{     SOURCE CODE files for Jetan 2.2:                                        }
{         JETAN.PAS -- The main Jetan program                                 }
{         JETANSPR.P -- Contains adaptation of Pavia's Turbo Sprite routines  }
{         JETANGRF.P -- Contains graphics for Jetan board and pieces          }
{         JETAN1.P  -- Contains definition of legal moves for pieces          }
{ --------------------------------------------------------------------------- }

   (* *************************************************************** *)
   (*                  TURBO SPRITES Library                          *)
   (*                              (c) Donald L. Pavia                *)
   (*         Ver 2.0              Department of Chemistry            *)
   (*      February 1986           Western Washington University      *)
   (*                              Bellingham, Washington 98225       *)
   (*                                                                 *)
   (* *************************************************************** *)

type  str14 = string[14];           { this is used for filenames it is long }
                                    { enough to include a drive designation }
                                    { like B:, but must be made longer if   }
                                    { you wish to include a pathname        }

      GrafScreen = record                                { definition of    }
                evenpixel : array[0..99,0..79] of byte;  { 320 x 200 screen }
                filler    : array[1..192] of byte;
                oddpixel  : array[0..99,0..79] of byte;
              end;

Image = array[1..128] of byte;
(*      Image = record                                 { definition of sprite }
                srows  : integer; scols  : integer;      {     dimensions }
                first  : array[0..9,0..5] of byte;       { 10 rows,6 cols }
                second : array[0..9,0..5] of byte;       { 10 rows,6 cols }
                future_use : array[0..3] of byte; { to pad out to 128 bytes   }
              end;                                { to allow use of
              blockread } *)

      SpriteTable = array[1..37] of Image;

var   ColorBuffer : GrafScreen absolute $B800:$0000;
      WorkBuffer  : GrafScreen;        { these are the display pages }

      Sprite : Image;                  { Sprite is the currently active one.}

      SpriteRows,Spritecols : integer;
{----------------------------------------------------------------------------}
procedure PutSpriteW (col,row : integer);      { direct display of sprite in }
                                               { hidden screen (workbuffer)  }
var   rowpos,colpos,i,j : integer;             {      notice the W           }

begin
     SpriteRows := Sprite[1]; SpriteCols := Sprite[3];

     if odd (row) then begin
       rowpos := (row-spriterows-1) shr 1;
       colpos := col shr 2 - (spritecols+1) shr 1;

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           WorkBuffer.oddpixel  [rowpos+i,colpos+j] := sprite[5+6*i+j];

       for i := 1 to spriterows+1 do
         for j := 0 to spritecols do
           WorkBuffer.evenpixel [rowpos+i,colpos+j] := sprite[65+6*(i-1)+j];
     end

     else begin
       rowpos := (row-spriterows) shr 1;
       colpos := col shr 2 - (spritecols+1) shr 1;

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           WorkBuffer.evenpixel [rowpos+i,colpos+j] := sprite[5+6*i+j];

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           WorkBuffer.oddpixel [rowpos+i,colpos+j] := sprite[65+6*i+j];
     end;

end; { procedure PutSpriteW (WorkBuffer) }
{----------------------------------------------------------------------------}
procedure XorSpriteW (col,row : integer);         { xor display of sprite }
                                                  { in hidden workbuffer  }
var   rowpos,colpos,i,j : integer;                {    notice the W       }

begin
     SpriteRows := Sprite[1]; SpriteCols := Sprite[3];

     if odd (row) then begin
       rowpos := (row-spriterows-1) shr 1;
       colpos := col shr 2 - (spritecols+1) shr 1;

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           if sprite[5+6*i+j] <> 0 then { do not xor with blank space }
           WorkBuffer.oddpixel  [rowpos+i,colpos+j] :=
            WorkBuffer.oddpixel [rowpos+i,colpos+j] xor sprite[5+6*i+j];

       for i := 1 to spriterows+1 do
         for j := 0 to spritecols do
         if sprite[65+6*(i-1)+j] <> 0 then { do not xor with blank space }
           WorkBuffer.evenpixel [rowpos+i,colpos+j] :=
            WorkBuffer.evenpixel [rowpos+i,colpos+j] xor sprite[65+6*(i-1)+j];
     end

     else begin
       rowpos := (row-spriterows) shr 1;
       colpos := col shr 2 - (spritecols+1) shr 1;

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           WorkBuffer.evenpixel [rowpos+i,colpos+j] :=
            WorkBuffer.evenpixel [rowpos+i,colpos+j] xor sprite[5+6*i+j];

       for i := 0 to spriterows do
         for j := 0 to spritecols do
           WorkBuffer.oddpixel [rowpos+i,colpos+j] :=
            WorkBuffer.oddpixel [rowpos+i,colpos+j] xor sprite[65+6*i+j];
     end;

end; { procedure XorSpriteW }
{----------------------------------------------------------------------------}




